import { Controller } from '@hotwired/stimulus';

export default class extends Controller {
  static targets = ['input', 'fileName', 'fileInfo', 'placeholder'];

  connect() {
    this.inputTarget.addEventListener('change', this.displayFileName.bind(this));
  }

  disconnect() {
    this.inputTarget.removeEventListener('change', this.displayFileName.bind(this));
  }

  selectFile() {
    this.inputTarget.click();
  }

  highlight(event) {
    event.preventDefault();
    this.element.classList.add('bg-gray-200');
  }

  dropFile(event) {
    event.preventDefault();
    this.element.classList.remove('bg-gray-200');
    const files = event.dataTransfer.files;
    this.inputTarget.files = files;
    this.displayFileName();
    this.inputTarget.dispatchEvent(new Event('change'));
  }

  displayFileName() {
    if (this.inputTarget.files.length > 0) {
      const file = this.inputTarget.files[0];
      this.fileNameTarget.textContent = file.name;
      this.fileInfoTarget.classList.remove('hidden');
      this.placeholderTarget.classList.add('hidden');
    }
  }

  removeFile(event) {
    event.stopPropagation();
    this.inputTarget.value = null;
    this.inputTarget.dispatchEvent(new Event('change'));
    this.fileInfoTarget.classList.add('hidden');
    this.placeholderTarget.classList.remove('hidden');
  }
}
