import { Controller } from '@hotwired/stimulus';

import SlimSelect from 'slimselect';
export default class extends Controller {
  connect() {
    this.form = this.element.closest('form');

    this.slimSelectInstance = new SlimSelect({
      select: this.element,
      settings: {
        placeholderText: '',
        searchText: 'Aucun résultat',
        searchPlaceholder: 'Rechercher',
      },
      events: {
        afterChange: info => {
          const event = new CustomEvent('valueChanged', { detail: { selectedValues: info } });
          this.element.dispatchEvent(event);
        },
        searchFilter: (option, search) => {
          const normalizedOption = this.normalizeText(option.text);
          const normalizedSearch = this.normalizeText(search);

          return normalizedOption.includes(normalizedSearch);
        },
      },
    });

    this.form.addEventListener('reset', () => {
      this.slimSelectInstance.setSelected([]);
    });
  }

  disconnect() {
    this.element.slim.destroy();
    this.form.removeEventListener('reset', this);
  }

  normalizeText(text) {
    return text
      .toLowerCase()
      .normalize('NFD')
      .replace(/[\u0300-\u036f]/g, '');
  }
}
