import { Controller } from '@hotwired/stimulus';

export default class extends Controller {
  connect() {
    this.isOpen = false;
    this.parentIds = this.element.dataset.parentIds;
    this.parentId = this.parentIds.split(',').pop();
    this.closeImg = this.element.dataset.closeImg;
    this.openImg = this.element.dataset.openImg;
    this.subrows = document.querySelectorAll(`tr[data-parent-id="${this.parentId}"]`);
    this.allSubrows = document.querySelectorAll(`tr[data-parent-ids*="${this.parentIds}"]`);
    this.element.addEventListener('click', this.toggleSubrows.bind(this));
    this.element.closest('table').addEventListener('subrows:close', this.closeIfParentClosed.bind(this));
  }

  disconnect() {
    this.element.removeEventListener('click', this.toggleSubrows.bind(this));
    this.element.removeEventListener('subrows:close', this.toggleSubrows.bind(this));

    this.allSubrows.forEach(subrow => {
      subrow.remove();
    });
  }

  toggleSubrows() {
    if (this.isOpen) {
      this.close();
      this.element.dispatchEvent(
        new CustomEvent('subrows:close', {
          bubbles: true,
          detail: { parentId: this.parentId },
        })
      );
    } else {
      this.open();
    }
  }

  close() {
    this.isOpen = false;
    this.subrows.forEach(subrow => {
      subrow.classList.add('hidden');
    });
    this.element.src = this.closeImg;
  }

  open() {
    this.isOpen = true;
    this.subrows.forEach(subrow => {
      subrow.classList.remove('hidden');
    });
    this.element.src = this.openImg;
  }

  closeIfParentClosed(event) {
    if (event.detail.parentId !== this.parentId && this.isOpen && this.parentIds.includes(event.detail.parentId)) {
      this.close();
    }
  }
}
