# frozen_string_literal: true

module SpiritHub
  class ApplicationController < ActionController::Base
    include Pundit::Authorization
    protect_from_forgery with: :exception
    before_action :try_to_set_user_from_token
    before_action :authenticate_user!

    after_action :verify_authorized
    after_action :verify_policy_scoped

    def authenticate_user!
      return redirect_to('/users/sign_in', alert: 'You need to sign up before continuing.') unless user_signed_in?

      @user = current_user
    end

    def auth_user_token
      request.headers['X-auth-user-token']
    end

    def set_user_favorites
      @user_favorites = Favorite.where(user_id: @user.id).pluck(:tagset_id).flatten.uniq.compact
    end

    private

    def try_to_set_user_from_token
      return unless auth_user_token

      user = User.find_by(authentication_token: auth_user_token)

      return unless user && user != warden.user

      warden.set_user(user)
    end

    def route_from_referer(referer)
      return nil if referer.blank?

      begin
        Rails.application.routes.recognize_path(URI(referer).path)
      rescue ActionController::RoutingError
        nil
      end
    end
  end
end
