# frozen_string_literal: true

module SpiritHub
  class SearchController < ApplicationController
    before_action :find_models, only: %i[index sort]
    before_action :set_seach_params, only: %i[index sort]

    include Datatablable

    def index
      skip_authorization
      skip_policy_scope

      respond_to(&:html)
    end

    def sort
      skip_authorization
      skip_policy_scope

      render partial: 'spirit_hub/search/table'
    end

    private

    def default_columns
      {
        type: { label: 'Type', sortable: true },
        cleaned_main_key: { label: 'Libellé', sortable: true },
        operation_title: { label: 'Opération', sortable: true },
        doc_file_size: { label: 'Poids', sortable: true },
        updated_at: { label: 'Dernière modification', sortable: true },
        last_current_user_name: { label: 'Provenance', sortable: true },
        informed: { label: 'Partagé', sortable: false },
        current_state: { label: 'Statut', sortable: true }
      }.freeze
    end

    def find_models
      scope = Tagset.where({ labels: 'hubdoc', is_tempalte: nil }.merge(search_params))
      @models = find_models_page(scope, default_sort_by: 'cleaned_main_key')
      @possible_states = ['À completer', 'Complet', 'À traiter']
    end

    def search_params
      permitted_params = params.require(:search_params).permit(:cleaned_main_key, :operation_id, :type, last_current_user_id: [], current_state: []).to_h

      permitted_params.each do |k, v|
        permitted_params[k] = v.reject(&:blank?)  if v.is_a?(Array)
        permitted_params.delete(k) if v.blank? || (v.is_a?(Array) && v.all?(&:blank?))
      end

      permitted_params = clean_params(permitted_params)
    end

    def set_seach_params
      @search_params = params[:search_params].to_unsafe_hash || {}
    end

    def clean_params(permitted_params)
      if permitted_params[:cleaned_main_key].present?
        permitted_params[:cleaned_main_key] = { '$regex' => /#{Regexp.escape(permitted_params[:cleaned_main_key]).accent_to_regexp}/i }
      end
      permitted_params[:current_state] = { '$in' => permitted_params[:current_state].compact } if permitted_params[:current_state].present?
      permitted_params[:last_current_user_id] = { '$in' => permitted_params[:last_current_user_id].compact } if permitted_params[:last_current_user_id].present?
      permitted_params
    end
  end
end
