# frozen_string_literal: true

require 'factory_bot_rails'
require 'database_cleaner/mongoid'

include FactoryBot::Syntax::Methods # rubocop:disable Style/MixinUsage
DatabaseCleaner.clean

FactoryBot.definition_file_paths << File.join(Rails.root, 'packages', 'spirit_hub', 'spec', 'factories')
FactoryBot.factories.clear
FactoryBot.find_definitions

password = 'Password1!'
admin = create(:user, roles: ['Administrator'], email: 'admin@test.com', password:)

def create_completeness_list(titles, tagset)
  titles.each do |title|
    create(:completeness, title:, tagset:, min_objects: 1)
  end
end

def create_directories_list(titles, tagset)
  admin = User.find_by(email: 'admin@test.com')
  titles.map do |title|
    id = create(:tagset_directory, title:, responsible: admin.id).id
    tagset.update(tagset_ids: tagset.tagset_ids.to_a + [id])
  end
end

operation = create(:tagset_directory, id: 'operation', title: 'Operation Carnot Lyon', responsible: admin.id)
maitrise_ouvrage = create(:tagset_directory, id: 'maitrise_ouvrage', title: 'Maitrise d\'ouvrage', responsible: admin.id)
operation.update(tagset_ids: operation.tagset_ids.to_a + [maitrise_ouvrage.id])
create_completeness_list(['Conformité'], maitrise_ouvrage)

terrain = create(:tagset_directory, title: 'Terrain', responsible: admin.id)
maitrise_ouvrage.update(tagset_ids: maitrise_ouvrage.tagset_ids.to_a + [terrain.id])

create_completeness_list(['Convention(s) collectivités'], terrain)

descriptif_foncier = create(:tagset_directory, title: 'Descriptif foncier', responsible: admin.id)
terrain.update(tagset_ids: terrain.tagset_ids.to_a + [descriptif_foncier.id])

create_completeness_list([
                           'Plan de situation',
                           'Etat des lieux initial (photos + plans + rapport)',
                           "Extrait cadastral - documents d'arpentage (plan géomètre)",
                           'Règlement applicable (PLU, POS, ZAC, Servitude, PPRI, MH, etc…)'
                         ], descriptif_foncier)

etudes_realisees = create(:tagset_directory, title: 'Etudes réalisées', responsible: admin.id)
terrain.update(tagset_ids: terrain.tagset_ids.to_a + [etudes_realisees.id])
create_completeness_list([
                           'Etude des sols préliminaire',
                           'Etude des sols complémentaire',
                           'Diagnostic pollution',
                           'Attestation(s) de dépollution'
                         ], etudes_realisees)

existant = create(:tagset_directory, title: 'Existant', responsible: admin.id)
etudes_realisees.update(tagset_ids: etudes_realisees.tagset_ids.to_a + [existant.id])
create_completeness_list([
                           'Rapport(s) amiante',
                           'Rapport(s) plomb',
                           'Etat(s) parasitaire(s)'
                         ], existant)

acquisition = create(:tagset_directory, title: 'Acquisition Propriétaire XX', responsible: admin.id)
terrain.update(tagset_ids: terrain.tagset_ids.to_a + [acquisition.id])
create_completeness_list([
                           'Etat locatif (bail, loyer, bilans,…)',
                           'Offre acceptée par le propriétaire',
                           'Promesse, avenant et annexes',
                           'Convention(s) propriétaire et-ou locataire (servitude, éviction,…)',
                           'Acte authentique'
                         ], acquisition)

create_directories_list([
                          'Projet',
                          'NF Logement',
                          'Financier',
                          'Copromotion et partenariat',
                          'Programme',
                          'Trésor public',
                          'Prestataires techniques',
                          'Référé préventif',
                          'Assurances ',
                          'Dossier commercial'
                        ], maitrise_ouvrage)

maitrise_execution = create(:tagset_directory, title: "Maîtrise d'œuvre d'exécution", responsible: admin.id)
operation.update(tagset_ids: operation.tagset_ids.to_a + [maitrise_execution.id])
create_directories_list([
                          "Lancement de l'exécution",
                          'Concessionnaires',
                          'Prestataires techniques',
                          "Appel d'offre - DCE",
                          'Dossier Marché',
                          "Corps d'état",
                          'Phase chantier',
                          'Réception des travaux'
                        ], maitrise_execution)
create_completeness_list([
                           'Assurance des entreprises',
                           'Opérations préalables à la réception (OPR)',
                           "Relations Maitrise d'ouvrage"
                         ], maitrise_execution)

juridique = create(:tagset_directory, title: 'Juridique et gestion des risques', responsible: admin.id)
operation.update(tagset_ids: operation.tagset_ids.to_a + [juridique.id])
create_directories_list([
                          'Contentieux Entreprises',
                          'Contentieux Clients',
                          'Contentieux Tiers',
                          'Déclarations de sinistres'
                        ], juridique)
create_completeness_list(["Tableau récapitulatif des contentieux de l'opération"], juridique)

carnot_lyon = create(:operation, title: 'Carnot Lyon')
Tagset.all.each do |tagset|
  tagset.update(operation_id: carnot_lyon.id)
end

create_list(:user, 10)

puts "Rails environment seeded with the following data: #{Rails.env}"
puts "Users: #{User.count}"
puts "Tagsets: #{Tagset.count}"
puts "Completeness: #{SpiritHub::Completeness.count}"
