import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';

let tagset_id;

Given('There is one doc', () => {
  cy.appFactories([['create', 'doc_favorite', 'with_file', 'with_current_user', 'with_informed', 'with_logs', { user_id: '1' }]]).then(records => {
    tagset_id = records[0].tagset_id;
  });
});

When('I click on the first doc', () => {
  cy.get('#model_table tbody tr:first td:last a:first').click();
});

When('I visit the doc url', () => {
  cy.visit(`/spirit_hub/tagsets/${tagset_id}/doc`);
});

When('I click on the next button', () => {
  cy.get('button[data-docviewer-target="nextPageButton"]').click();
});

When('I click on the previous button', () => {
  cy.get('button[data-docviewer-target="previousPageButton"]').click();
});

When('I click on the page 5', () => {
  cy.get('button[data-docviewer-target="navigationButtons"]').eq(4).click();
});

When('I zoom in', () => {
  cy.get('button[data-docviewer-target="zoomInButton"]').click();
});

When('I zoom out', () => {
  cy.get('button[data-docviewer-target="zoomOutButton"]').click();
});

When('I zoom all the way in', () => {
  for (let i = 0; i < 10; i++) {
    cy.get('button[data-docviewer-target="zoomInButton"]').click({ force: true });
  }
});

When('I zoom all the way out', () => {
  for (let i = 0; i < 10; i++) {
    cy.get('button[data-docviewer-target="zoomOutButton"]').click({ force: true });
  }
});

Then('I should be on the first page', () => {
  cy.get('button[data-docviewer-target="previousPageButton"]').should('be.disabled');
  cy.get('button[data-docviewer-target="nextPageButton"]').should('not.be.disabled');

  cy.get('button[data-docviewer-target="navigationButtons"]').eq(0).should('have.class', 'bg-primary');
});

Then('I should be on the second page', () => {
  cy.get('button[data-docviewer-target="previousPageButton"]').should('not.be.disabled');
  cy.get('button[data-docviewer-target="nextPageButton"]').should('not.be.disabled');

  cy.get('button[data-docviewer-target="navigationButtons"]').eq(1).should('have.class', 'bg-primary');
});

Then('I should be on the fifth page', () => {
  cy.get('button[data-docviewer-target="previousPageButton"]').should('not.be.disabled');
  cy.get('button[data-docviewer-target="nextPageButton"]').should('be.disabled');

  cy.get('button[data-docviewer-target="navigationButtons"]').eq(4).should('have.class', 'bg-primary');
});

Then('The images should be at {string} zoom', zoom => {
  cy.get('img[data-docviewer-target="pages"]').first().should('have.attr', 'style', `width: ${zoom};`);
});

Then('The zoom in button should be disabled', () => {
  cy.get('button[data-docviewer-target="zoomInButton"]').should('be.disabled');
});

Then('The zoom out button should be disabled', () => {
  cy.get('button[data-docviewer-target="zoomOutButton"]').should('be.disabled');
});
