import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';

// Counts categories
Given('There is some docs to count', () => {
  cy.appFactories([['create_list', 'tagset_doc', 1, { responsible: '1', tagset_ids: [] }]]);
  cy.appFactories([['create_list', 'tagset_doc', 2, { accounted: ['1'], percent: 20, parent_tagset_ids: ['A'] }]]);
  cy.appFactories([['create_list', 'tagset_doc', 3, { informed: ['other_user'] }]]);
});

Then('I should see the right counts', () => {
  cy.get('.category_count').eq(0).contains(1);
  cy.get('.category_count').eq(1).contains(2);
  cy.get('.category_count').eq(2).contains(3);
});

When('I click on the first category', () => {
  cy.get('.category_count').eq(0).click();
});

When('I click on the second category', () => {
  cy.get('.category_count').eq(1).click();
});

// Right colums
Given('There is one favorites with all columns', () => {
  cy.appFactories([['create', 'doc_favorite', 'with_file', 'with_current_user', 'with_informed', { user_id: '1' }]]);
});

Then('I should see the right columns', () => {
  cy.get('#model_table td').eq(2).contains('Doc Title 1');
  cy.get('#model_table td').eq(3).contains('298.15 Ko');
  cy.get('#model_table td').eq(4).contains(new Date().toLocaleDateString('fr-FR'));
  cy.get('#model_table td').eq(5).find('div').should('exist');
  cy.get('#model_table td').eq(6).find('img').should('exist');
  cy.get('#model_table td').eq(7).contains('À traiter');
  cy.get('#model_table td').eq(8).find('a').should('exist');
});

// Sort
Given('There is some docs and tagset', () => {
  cy.appFactories([['create_list', 'doc_favorite', 5, { user_id: '1' }]]);
  cy.appFactories([['create_list', 'tagset_favorite', 5, { user_id: '1' }]]);
});

When('I click on the title sort', () => {
  cy.get('#model_table th').eq(2).find('a').click();
});

When('I click on the status sort', () => {
  cy.wait(100);
  cy.get('#model_table th').eq(7).find('a').click();
});

Then('The table is sorted by title desc', () => {
  cy.get('#model_table td').eq(2).contains('Tagset Title 5');
});

Then('The table is sorted by title asc', () => {
  cy.get('#model_table td').eq(2).contains('Doc Title 1');
});

Then('The table is sorted by status asc', () => {
  cy.get('#model_table td').eq(7).contains('À traiter');
});

Then('The table is sorted by status desc', () => {
  cy.get('#model_table td').eq(7).contains('Complet');
});

// Per page and pagination
Given('There is a lot of docs and tagset', () => {
  cy.appFactories([['create_list', 'doc_favorite', 20, { user_id: '1' }]]);
  cy.appFactories([['create_list', 'tagset_favorite', 20, { user_id: '1' }]]);
});

When('I go to page 5', () => {
  cy.get('.pagination a').contains('5').click();
});

Then('I am on page 1 and i see 8 pages', () => {
  cy.get('#model_table tbody tr').should('have.length', 5);
  cy.get('.pagination a').eq(0).should('have.class', 'cursor-not-allowed');
  cy.get('.pagination a.bg-primary').contains('1');
  cy.get('.pagination a').eq(6).contains('...');
  cy.get('.pagination a').eq(7).contains('8');
});

Then('I am on page 5 and i see 8 pages', () => {
  cy.get('#model_table tbody tr').should('have.length', 5);
  cy.get('.pagination a').eq(0).should('not.have.class', 'cursor-not-allowed');

  cy.get('.pagination a').eq(2).contains('...');
  cy.get('.pagination a.bg-primary').contains('5');
  cy.get('.pagination a').eq(8).contains('...');
  cy.get('.pagination a').eq(9).contains('8');
});

Then('I am on page 1 and i see 2 pages', () => {
  cy.get('#model_table tbody tr').should('have.length', 20);
  cy.get('.pagination a').eq(0).should('have.class', 'cursor-not-allowed');
  cy.get('.pagination a.bg-primary').contains('1');
  cy.get('.pagination a').eq(2).contains('2');
});

// Logs
Given('There is a doc with logs and a tagset without logs', () => {
  cy.appFactories([['create', 'doc_favorite', 'with_logs', { user_id: '1' }]]);
  cy.appFactories([['create', 'tagset_favorite', { user_id: '1' }]]);
});

When('I click on the logs button on the doc', () => {
  cy.get('#model_table tbody tr:first td:eq(4)').find('button').click();
});

Given('There is some doc and tagset of different types', () => {
  cy.appFactories([['create', 'doc_favorite', 'with_file', 'with_current_user', 'with_informed', { user_id: '1' }]]);
  cy.appFactories([['create', 'doc_favorite', 'with_current_user', 'with_informed', { user_id: '1' }]]);
  cy.appFactories([['create_list', 'tagset_favorite', 5, { user_id: '1' }]]);
});

When('I click on the logs button on the tagset', () => {
  cy.get('#model_table tbody tr:eq(1) td:eq(4)').find('button').click();
});

When('I click outside the logs', () => {
  cy.get('body h1').first().click();
});

When('I click on the close log button', () => {
  cy.get('#model_table tbody tr:eq(1) td:eq(4) #tagset_logs').find('button').click();
});

Then('I should see the doc logs', () => {
  cy.get('.logs').should('be.visible');
  cy.get('.log').should('have.length', 21);
});

Then('I should not see the doc logs', () => {
  cy.get('.logs').should('not.be.visible');
  cy.get('.log').should('have.length', 21);
});

Then('I should see the tagset logs', () => {
  cy.get('#tagset_logs p').contains('Aucun log');
});

Then('I should not see the tagset logs', () => {
  cy.get('#tagset_logs').should('not.be.visible');
});

// Remove from favorite
When('I remove the first and last element from favorite', () => {
  cy.get('#model_table tbody tr:first td:eq(0) a').click();
  cy.get('#model_table tbody tr:last td:eq(0) a').click();
});

Then('The favorites are removed', () => {
  cy.get('#model_table tbody tr').should('have.length', 8);
});

When('I remove the first and last element from favorite in the grid view', () => {
  cy.get('a').filter('[href*="/remove"]').first().click();
  cy.get('a').filter('[href*="/remove"]').last().click();
});

Then('The other favorites are removed', () => {
  cy.get('#model_table tbody tr').should('have.length', 6);
});

// Go to doc_view
Given('There is one tagset_doc with file', () => {
  cy.appFactories([['create', 'doc_favorite', 'with_file', 'with_current_user', 'with_informed', { user_id: '1' }]]);
});

When('I click on the first doc view link', () => {
  cy.get('a').filter('[href*="/doc"]').first().click({ force: true });
});

Then('I should be on the doc_view page', () => {
  cy.url().should('include', '/doc');
});

// Go to directory_view
Given('There is one taget_directory', () => {
  cy.appFactories([['create', 'tagset_favorite', { user_id: '1' }]]);
});

When('I click on the first directory view link', () => {
  cy.get('a').filter('[href*="/index"]').first().click({ force: true });
});

Then('I should be on the directory_view page', () => {
  cy.url().should('include', '/index');
});

// Delete object
When('I delete the first and last element', () => {
  cy.get('#model_table tbody tr:first td:last button:last').click();
  cy.get('#model_table tbody tr:last td:last button:last').click();
});

Then('The element are deleted', () => {
  cy.get('#model_table tbody tr').should('have.length', 8);
});

When('I delete the first and last element in the grid view', () => {
  cy.get('.remove-button').first().click({ force: true });
  cy.get('.remove-button').last().click({ force: true });
});

Then('The other element are deleted', () => {
  cy.get('#model_table tbody tr').should('have.length', 6);
});
