import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';

Given('I am guest', () => {
  cy.app('clean');
});

Given('I am a logged in admin', () => {
  cy.app('clean');
  cy.appFactories([
    [
      'create',
      'user',
      {
        id: '1',
        email: 'admin@test.com',
        password: 'Password1!',
        roles: ['Administrator'],
        authentication_token: 'johns_token',
      },
    ],
  ]);
  cy.addAuthHeader('johns_token');
});

When('I visit the url {word}', url => {
  let headers = {};
  if (Cypress.env('auth-user-token')) {
    headers = {
      'X-auth-user-token': Cypress.env('auth-user-token'),
    };
  }
  cy.visit({
    url: `${Cypress.config().baseUrl}${url}`,
    failOnStatusCode: false,
    headers,
  });
});

Then('The url should match {string}', urlPattern => {
  const dynamicUrl = urlPattern.replace(/\*/g, '.*');
  const regex = new RegExp(`^${Cypress.config().baseUrl}${dynamicUrl}$`);
  cy.url().should('match', regex);
});

Then('The url is {word}', url => {
  cy.url().should('eq', `${Cypress.config().baseUrl}${url}`);
});

When('I select {string} per page', text => {
  cy.get('select[name="per_page"]').select(text);
});

When('I remove the title column', () => {
  cy.get('#column-selector button').click();
  cy.wait(100);
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
});

When('I readd the title column', () => {
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
});

Then('I see {string} columns', n => {
  cy.get('#model_table tbody tr:first td').should('have.length', n);
});

When('I click on the grid view button', () => {
  cy.get('#view-type-selector button:first').click();
});

When('I click on the table view button', () => {
  cy.get('#view-type-selector button:last').click();
});

Then('I see the grid view', () => {
  cy.get('#model_table .grid').should('exist');
});

Then('I see the table view', () => {
  cy.get('#model_table table').should('exist');
});

// Sort
Then('I can sort', () => {
  cy.get('#model_table td').eq(3).contains('Doc Title 1');
  cy.get('#model_table th').eq(3).find('a').click();
  cy.get('#model_table td').eq(3).contains('Doc Title 5');
});

// Per page and pagination
Then('I can paginate', () => {
  cy.get('select[name="per_page"]').select('5');
  cy.get('#model_table tbody tr').should('have.length', 6);
  cy.get('.pagination a').eq(0).should('have.class', 'cursor-not-allowed');
  cy.get('.pagination a.bg-primary').contains('1');
  cy.get('.pagination a').eq(4).contains('4');
  cy.get('.pagination a').contains('3').click();
  cy.get('#model_table tbody tr').should('have.length', 6);
  cy.get('.pagination a').eq(0).should('not.have.class', 'cursor-not-allowed');
  cy.get('.pagination a.bg-primary').contains('3');
  cy.get('.pagination a').eq(4).contains('4');
});

// Colums selector
Then('I can select columns', () => {
  cy.get('#model_table tbody tr:first td').should('have.length', 10);
  cy.get('#column-selector button').click();
  cy.wait(100);
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
  cy.get('#model_table tbody tr:first td').should('have.length', 9);
  cy.get('#column-selector label[for="columns_cleaned_main_key"]').click();
  cy.get('#model_table tbody tr:first td').should('have.length', 10);
});

// View selector
Then('I can switch between views', () => {
  cy.get('#model_table table').should('exist');
  cy.get('#view-type-selector button:first').click();
  cy.get('#model_table .grid').should('exist');
  cy.get('#view-type-selector button:last').click();
  cy.get('#model_table table').should('exist');
});

//  Add remove from favorites
Then('I can add and remove from favorites', () => {
  cy.get('#model_table td').eq(0).find('button').click();
  cy.get('#model_table td').eq(0).find('svg').should('have.class', 'text-yellow-500');
  cy.get('#model_table td').eq(0).find('button').click();
  cy.get('#model_table td').eq(0).find('svg').should('have.class', 'text-gray-200');
  cy.get('#view-type-selector button:first').click();
  cy.get('.card form').filter('[action*="/add_favorite"]').first().submit();
  cy.get('.card').eq(0).find('svg').should('have.class', 'text-yellow-500');
  cy.get('.card form').filter('[action*="/remove_favorite"]').first().submit();
  cy.get('.card').eq(0).find('svg').should('have.class', 'text-gray-200');
});

// Remove
Then('I can remove element', () => {
  cy.get('#model_table tbody tr:first .remove-button').click();
  cy.get('#model_table tbody tr').should('have.length', 5);
  cy.get('#view-type-selector button:first').click();
  cy.get('.grid .remove-button').first().click({ force: true });
  cy.get('#view-type-selector button:last').click();
  cy.get('#model_table tbody tr').should('have.length', 4);
});

// Bulk delete
Then('I can bulk delete', () => {
  cy.get('#model_table tbody tr').eq(0).find('td:eq(1) input').check();
  cy.get('#model_table tbody tr').eq(1).find('td:eq(1) input').check();
  cy.get('#model_table button[formaction="/spirit_hub/tagsets/bulk_delete"]').click();
  cy.get('#model_table tbody tr').should('have.length', 4);
  cy.get('#view-type-selector button:first').click();
  cy.get('.grid>div:first input:first').check({ force: true });
  cy.get('.grid>div:eq(2) input:first').check({ force: true });
  cy.get('#model_table button[formaction="/spirit_hub/tagsets/bulk_delete"]').click();
  cy.get('#view-type-selector button:last').click();
  cy.get('.grid>div').should('have.length', 1);
});

Then('I can share', () => {
  cy.get('div[id*="informed_badges"]')
    .first()
    .then($div => {
      const before_img_length = $div.find('img').length;

      cy.get('#model_table tbody tr:first form').filter('[action*="/share_form"]').first().submit();
      cy.get('select[name="to[]"]').next('div').click();
      cy.get('.ss-list > div').first().click();
      cy.get('form[action*="/share"]').first().submit();
      cy.get('div[id*="informed_badges"]')
        .first()
        .find('img')
        .should('have.length', before_img_length + 1);
      cy.get('#view-type-selector button:first').click();
      cy.get('.grid form[action*="/share_form"]').eq(1).submit();
      cy.get('select[name="to[]"]').next('div').click();
      cy.get('.ss-list > div').first().click();
      cy.get('form[action*="/share"]').first().submit();
      cy.get('div[id*="informed_badges"]')
        .eq(1)
        .find('img')
        .should('have.length', before_img_length + 1);
    });
});
