# frozen_string_literal: true

require 'faker'

FactoryBot.define do
  factory :tagset_directory, class: 'Tagset' do
    title { FactoryBot.generate(:generate_unique_tagset_title) }
    last_current_user_name { %w[Jean Paul Jacques Sophie].sample }
    end_date { Date.today + rand(1..30) }
    type { 'directory' }
    labels { ['hubdoc'] }
    operation_id { '123456' }

    trait :with_logs do
      after(:create) do |tagset|
        tagset.logs.create(action: 'Crée', user_id: tagset.responsible)
        20.times do
          tagset.logs.create(action: 'Modifier', user_id: tagset.responsible)
        end
      end
    end
  end

  factory :tagset_doc, class: 'Tagset' do
    title { FactoryBot.generate(:generate_unique_doc_title) }
    last_current_user_name { %w[Jean Paul Jacques Sophie].sample }
    end_date { Date.today + rand(1..30) }
    type { 'document' }
    labels { ['hubdoc'] }
    current_state { 'À traiter' }
    operation_id { '123456' }

    trait :with_file do
      after(:create) do |tagset,|
        doc_data = {
          title: tagset.title,
          responsible: tagset.responsible
        }

        doc = FactoryBot.create(:doc, doc_data)
        tagset.update(doc_ids: [doc.id])

        File.open(Rails.root.join('packages/spirit_hub/spec/fixtures/files/pdf_sample.pdf'), 'rb') do |f|
          doc.type = 'file'
          doc.store(f)
          doc.save
        end

        Rails.logger.info "DEBUG: #{DocPreviewWorker.new.perform(doc.id.to_s)}"
        tagset.update_doc_fields
        tagset.save
      end
    end

    trait :with_informed do
      after(:create) do |tagset|
        users = FactoryBot.create_list(:user, rand(1..5))
        tagset.update(informed: users.map(&:id))
      end
    end

    trait :with_logs do
      after(:create) do |tagset|
        tagset.logs.create(action: 'Crée', user_id: tagset.responsible)
        20.times do
          tagset.logs.create(action: 'Modifier', user_id: tagset.responsible)
        end
      end
    end
  end

  sequence :generate_unique_doc_title do |n|
    "Doc Title #{n}"
  end

  sequence :generate_unique_tagset_title do |n|
    "Tagset Title #{n}"
  end
end
