# frozen_string_literal: true

require 'rails_helper'

RSpec.describe 'Create Users & Groups script' do
  let!(:admin) { create(:user, email: 'support@plugandwork.fr') }
  let(:script_path) { SpiritHub::Engine.root.join('lib/scripts/create_users_groups.rb') }
  let(:entry_csv) { create(:doc, title: 'entry_file_create_users_groups.csv') }
  let!(:group_all_users) { create(:group, name: 'Tous les utilisateurs') }
  let!(:tagset_batch) { create(:tagset, title: "Création des utilisateurs et groupes à partir d'un fichier") }

  before do
    entry_csv.blob = File.read(SpiritHub::Engine.root.join('lib/scripts/entry_file_create_users_groups.csv'))
    entry_csv.save!
  end

  subject { load script_path }

  it 'works' do
    subject
    expect(User.count).to eq(13)
    rapport_csv = Doc.find_by(title: /Rapport des utilisateurs - groupes créés/)

    expect(rapport_csv).to be_present
    user = User.find_by(email: 'grobaut@plugandwork.net')
    expect(user.login).to eq('grobaut@plugandwork.net')
    expect(user.first_name).to eq('Geoffroy')
    expect(user.last_name).to eq('ROBAUT')
    expect(user.trusted).to eq(true)
    expect(user.groups.count).to eq(1)
    expect(rapport_csv.data).to include('1 existing groups (Tous les utilisateurs) added to user')
  end

  context 'when the user already exists' do
    let!(:user) { create(:user, email: 'grobaut@plugandwork.net', first_name: 'Bob', last_name: 'Sponge', trusted: false, login: 'spongebob') }

    it 'update user information' do
      subject
      user.reload
      expect(user.login).to eq('grobaut@plugandwork.net')
      expect(user.first_name).to eq('Geoffroy')
      expect(user.last_name).to eq('ROBAUT')
      expect(user.trusted).to eq(true)
      expect(user.groups.count).to eq(1)
    end
  end

  it 'when the group does not exist' do
    group_all_users.destroy
    subject
    user = User.find_by(email: 'grobaut@plugandwork.net')
    expect(user.groups.count).to eq(1)
    rapport_csv = Doc.find_by(title: /Rapport des utilisateurs - groupes créés/)
    expect(rapport_csv.data).to include('1 new groups (Tous les utilisateurs) added to user')
  end
end
